@@ -1,6 +1,6 @@
 #!/usr/bin/env tclsh
 
-if {[catch {package require Tcl 8.6} msg]} {
+if {[catch {package require Tcl 8.6-} msg]} {
     puts stderr "ERROR: $msg"
     puts stderr "If running this script from 'make html', set the\
 	NATIVE_TCLSH environment\nvariable to point to an installed\
@@ -470,6 +470,7 @@ proc plus-pkgs {type args} {
 		continue
 	    }
 	}
+	set dir [string trimright $dir "0123456789-."]
 	switch $type {
 	    n {
 		set title "$name Package Commands"
@@ -647,10 +648,12 @@ try {
 	append appdir "$tkdir"
     }
 
+    apply {{} {
+    global packageBuildList tcltkdir tcldir build_tcl
 
     # When building docs for Tcl, try to build docs for bundled packages too
     set packageBuildList {}
-    if  {$build_tcl} {
+    if {$build_tcl} {
 	set pkgsDir [file join $tcltkdir $tcldir pkgs]
 	set subdirs [glob -nocomplain -types d -tails -directory $pkgsDir *]
 
@@ -664,7 +667,11 @@ try {
 
 	    # ... but try to extract (name, version) from subdir contents
 	    try {
-		set f [open [file join $pkgsDir $dir configure.in]]
+		try {
+		    set f [open [file join $pkgsDir $dir configure.in]]
+		} trap {POSIX ENOENT} {} {
+		    set f [open [file join $pkgsDir $dir configure.ac]]
+		}
 		foreach line [split [read $f] \n] {
 		    if {2 == [scan $line \
 			    { AC_INIT ( [%[^]]] , [%[^]]] ) } n v]} {
@@ -693,7 +700,8 @@ try {
 		foreach line [split [read $f] \n] {
 		    if {[string trim $line] eq ""} continue
 		    if {[string match #* $line]} continue
-		    lappend packageDirNameMap {*}$line
+		    lassign $line dir name
+		    lappend packageDirNameMap $dir $name
 		}
 	    } finally {
 		close $f
@@ -714,6 +722,7 @@ try {
 	    lset packageBuildList $idx+1 [dict get $packageDirNameMap $n]
 	}
     }
+    }}
 
     #
     # Invoke the scraper/converter engine.
