@@ -203,7 +203,7 @@ proc uni::main {} {
  * to the same alternate page number.
  */
 
-static CONST unsigned short pageMap\[\] = {"
+static const unsigned short pageMap\[\] = {"
     set line "    "
     set last [expr {[llength $pMap] - 1}]
     for {set i 0} {$i <= $last} {incr i} {
@@ -234,7 +234,7 @@ static CONST unsigned short pageMap\[\] = {"
  * set of character attributes.
  */
 
-static CONST unsigned char groupMap\[\] = {"
+static const unsigned char groupMap\[\] = {"
     set line "    "
     set lasti [expr {[llength $pages] - 1}]
     for {set i 0} {$i <= $lasti} {incr i} {
@@ -272,12 +272,13 @@ static CONST unsigned char groupMap\[\] = {"
  *				 100 = subtract delta for title/upper
  *				 101 = sub delta for upper, sub 1 for title
  *				 110 = sub delta for upper, add delta for lower
+ *				 111 = subtract delta for upper
  *
  * Bits 8-31	Case delta: delta for case conversions.  This should be the
  *			    highest field so we can easily sign extend.
  */
 
-static CONST int groups\[\] = {"
+static const int groups\[\] = {"
     set line "    "
     set last [expr {[llength $groups] - 1}]
     for {set i 0} {$i <= $last} {incr i} {
@@ -309,10 +310,14 @@ static CONST int groups\[\] = {"
 		}
 	    }
 	} elseif {$toupper} {
-	    # subtract delta for upper, add delta for lower
-	    set case 6
 	    set delta $toupper
-	    if {$tolower != $toupper} {
+	    if {$tolower == $toupper} {
+		# subtract delta for upper, add delta for lower
+		set case 6
+	    } elseif {!$tolower} {
+		# subtract delta for upper
+		set case 7
+	    } else {
 		error "New case conversion type needed: $toupper $tolower $totitle"
 	    }
 	} elseif {$tolower} {
@@ -396,7 +401,11 @@ enum {
  * Unicode character tables.
  */
 
-#define GetUniCharInfo(ch) (groups\[groupMap\[pageMap\[((ch) & 0xffff) >> OFFSET_BITS\] | ((ch) & ((1 << OFFSET_BITS)-1))\]\])
+#if TCL_UTF_MAX > 3
+#   define GetUniCharInfo(ch) (groups\[groupMap\[pageMap\[((ch) & 0x1fffff) >> OFFSET_BITS\] | ((ch) & ((1 << OFFSET_BITS)-1))\]\])
+#else
+#   define GetUniCharInfo(ch) (groups\[groupMap\[pageMap\[((ch) & 0xffff) >> OFFSET_BITS\] | ((ch) & ((1 << OFFSET_BITS)-1))\]\])
+#endif
 "
 
     close $f
